#include "menu.h"
#include "civ.h"
#include "tiles.h"
#include "draw.h"
#include "tools.h"
#include "undo.h"
#include "settings.h"
#include "overview.h"

#include <registry.h>
#include <game.h>
#include <map.h>

#include <string.h>
#include <stdlib.h>


typedef void (* MenuFunction)(void);

typedef struct _myMenuData
 {
  char * name;
  MenuFunction func;
 } MyMenuData;

void new_file(void);
void open_file(void);
void save_file(void);
void save_file_as(void);

/* File menu */
MyMenuData menu_file_names[]=
 {
  {"File",             (MenuFunction) NULL},
  {"New",              (MenuFunction) new_file},
  {"Open",             (MenuFunction) open_file},
  {"Save",             (MenuFunction) save_file},
  {"Save as...",       (MenuFunction) save_file_as},
  {(char *) SEPARATOR, (MenuFunction) NULL},
  {"Quit",             (MenuFunction) gtk_main_quit},
  {NULL, NULL}
 };

/* Edit menu */
MyMenuData menu_edit_names[]=
 {
  {"Edit",             (MenuFunction) NULL},
  {"Undo",             (MenuFunction) do_undo},
  {(char *) SEPARATOR, (MenuFunction) NULL},
  {"Cut",              (MenuFunction) cut},
  {"Copy",             (MenuFunction) copy},
  {"Paste",            (MenuFunction) paste},
  {NULL, NULL}
 };

/* Paint menu */
MyMenuData menu_paint_names[]=
 {
  {"Paint",            (MenuFunction) NULL},
  {"Tile",             (MenuFunction) tile},
  {"Line",             (MenuFunction) line},
  {"Rectangle",        (MenuFunction) rectangle},
  {(char *) SEPARATOR, (MenuFunction) NULL},
  {"Special",          (MenuFunction) special},
  {"Start Position",   (MenuFunction) startpos},
  {NULL, NULL}
 };

/* Extra menu */
MyMenuData menu_extras_names[]=
 {
  {"Extras",               (MenuFunction) NULL},
  {"Map and game options", (MenuFunction) create_settings},
  {"Clear all specials",   (MenuFunction) clear_specials},
  {NULL, NULL}
 };

/* View menu */
MyMenuData menu_view_names[]=
 {
  {"View",                       (MenuFunction) NULL},
  {"\tShow Toolbox",             (MenuFunction) show_tools},
  {"\tShow Overview",            (MenuFunction) show_ov},
  {(char *) SEPARATOR,           (MenuFunction) NULL},
  {"\tEnable tooltips",          (MenuFunction) show_tips},
  {(char *) SEPARATOR,           (MenuFunction) NULL},
  {"\tTransparent flags",        (MenuFunction) solid_bg},
  {"\tUnits on start positions", (MenuFunction) startpos_units},
/*
  {"\tShow diagonal roads",      (MenuFunction) diagonal_roads},
*/
  {NULL, NULL}
 };


char *filename=NULL;

GtkWidget *toolmi, *overmi;
GtkWidget *spinnerx;
GtkWidget *spinnery;

void create_menu(GtkWidget *widget, MyMenuData *menu);

void save_ok(GtkWidget *widget, GtkFileSelection *fs);
void fs_cancel(GtkWidget *widget, GtkFileSelection *fs);
void scenario_toggled(GtkWidget *widget, gpointer scenario);


/****************************************************************************
create all menus
*****************************************************************************/
void create_menus(GtkWidget *widget)
 {
  GtkWidget *menu_bar;

  menu_bar=gtk_menu_bar_new();
  create_menu(menu_bar, menu_file_names);
  create_menu(menu_bar, menu_edit_names);
  create_menu(menu_bar, menu_paint_names);
  create_menu(menu_bar, menu_extras_names);
  create_menu(menu_bar, menu_view_names);

  gtk_box_pack_start(GTK_BOX(widget), menu_bar, FALSE, FALSE, 1);
  gtk_widget_show(menu_bar);
} /* create_menus() */


/****************************************************************************
create a menu from the appropriate MenuData structure
*****************************************************************************/
void create_menu(GtkWidget *menu_bar, MyMenuData *data)
 {
  GtkWidget *menu;
  GtkWidget *menu_item;
  GtkWidget *root_menu;
  gint i;

  menu=gtk_menu_new();
  root_menu=gtk_menu_item_new_with_label(data[0].name);
  gtk_widget_show(root_menu);

  for(i=1; data[i].name != NULL; i++)
   {
    if(data[i].name==(char *)SEPARATOR)
     {
      menu_item = gtk_menu_item_new();
     }
    else
     {
      if(data[i].name[0]=='\t')
       {
	menu_item=gtk_check_menu_item_new_with_label(data[i].name+1);
        if(data[i].name[1]=='S')
         {
          if(data[i].name[6]=='T')
            toolmi=menu_item;
          else if(data[i].name[6]=='O')
            overmi=menu_item;
         }
	gtk_check_menu_item_set_show_toggle(GTK_CHECK_MENU_ITEM(menu_item),
        				    TRUE);
	gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(menu_item),TRUE);
       }
      else
	menu_item=gtk_menu_item_new_with_label(data[i].name);
      if(data[i].func != NULL)
       {
	gtk_signal_connect(GTK_OBJECT(menu_item), "activate",
			   GTK_SIGNAL_FUNC(data[i].func), NULL);
       }
     }
    gtk_menu_append(GTK_MENU(menu), menu_item);
    gtk_widget_show(menu_item);
   }

  gtk_menu_item_set_submenu(GTK_MENU_ITEM(root_menu), menu);
  gtk_menu_bar_append(GTK_MENU_BAR(menu_bar), root_menu);

 } /* create_menu() */


/****************************************************************************
button callbacks for mapsize dialog
*****************************************************************************/
void mapsize_ok(GtkWidget *widget, GtkWidget *dialog)
 {
  gint x,y;

  x=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinnerx));
  y=gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(spinnery));

  gtk_widget_destroy(widget);

  create_new_map(x,y);
  init_undo();

  draw_everything(FALSE);
  refresh_screen();
 } /* mapsize_ok()*/

void mapsize_cancel(GtkWidget *widget, GtkWidget *dialog)
 {
  gtk_widget_destroy(widget);
 } /* mapsize_cancel*/


/****************************************************************************
create a new game with default values
*****************************************************************************/
void new_file(void)
 {
  GtkWidget *mapsizewin;
  GtkWidget *table;
  GtkWidget *vbox;
  GtkWidget *hbox;
  GtkWidget *button;
  GtkWidget *label;
  GtkAdjustment *adj;

  mapsizewin=gtk_window_new(GTK_WINDOW_TOPLEVEL);
  gtk_window_set_title(GTK_WINDOW(mapsizewin), "Create new map");
  gtk_window_set_modal(GTK_WINDOW(mapsizewin), TRUE);
  gtk_signal_connect_object(GTK_OBJECT(mapsizewin), "delete_event",
		     GTK_SIGNAL_FUNC(mapsize_cancel), GTK_OBJECT(mapsizewin));

  vbox=gtk_vbox_new(FALSE, 5);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 10);
  gtk_container_add(GTK_CONTAINER(mapsizewin), vbox);

  table=gtk_table_new(2, 3, FALSE);
  gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);

  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_table_attach(GTK_TABLE(table), vbox, 0, 1, 0, 1,
		   GTK_FILL, GTK_FILL, 5, 5);

  label=gtk_label_new("xsize");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

  adj=(GtkAdjustment *)gtk_adjustment_new(MAP_DEFAULT_WIDTH,
					  MAP_MIN_WIDTH, MAP_MAX_WIDTH,
					  1.0, 5.0, 0.0);
  spinnerx=gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinnerx), TRUE);
  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinnerx),
				  GTK_SHADOW_ETCHED_IN);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinnerx), TRUE);
  gtk_tooltips_set_tip(tooltips, spinnerx, "Map width in squares", NULL);
  gtk_box_pack_start(GTK_BOX(vbox), spinnerx, FALSE, TRUE, 0);


  vbox=gtk_vbox_new(FALSE, 0);
  gtk_container_set_border_width(GTK_CONTAINER(vbox), 5);
  gtk_table_attach(GTK_TABLE(table), vbox, 1, 2, 0, 1,
		   GTK_FILL, GTK_FILL, 5, 5);

  label=gtk_label_new("ysize");
  gtk_misc_set_alignment(GTK_MISC(label), 0, 0.5);
  gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);

  adj=(GtkAdjustment *)gtk_adjustment_new(MAP_DEFAULT_HEIGHT,
					  MAP_MIN_HEIGHT, MAP_MAX_HEIGHT,
					  1.0, 5.0, 0.0);
  spinnery=gtk_spin_button_new(adj, 0, 0);
  gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spinnery), TRUE);
  gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spinnery),
				  GTK_SHADOW_ETCHED_IN);
  gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(spinnery), TRUE);
  gtk_tooltips_set_tip(tooltips, spinnery, "Map height in squares", NULL);
  gtk_box_pack_start(GTK_BOX(vbox), spinnery, FALSE, TRUE, 0);

  hbox=gtk_hseparator_new();
  gtk_table_attach(GTK_TABLE(table), hbox, 0, 2, 1, 2,
		   GTK_FILL, GTK_FILL, 0, 0);

  button=gtk_button_new_with_label("OK");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(mapsize_ok),
			    GTK_OBJECT(mapsizewin));
  gtk_table_attach(GTK_TABLE(table), button, 0, 1, 2, 3,
		   GTK_FILL, GTK_FILL, 5, 5);

  button=gtk_button_new_with_label("Cancel");
  gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
			    GTK_SIGNAL_FUNC(mapsize_cancel),
			    GTK_OBJECT(mapsizewin));
  gtk_table_attach(GTK_TABLE(table), button, 1, 2, 2, 3,
		   GTK_FILL, GTK_FILL, 5, 5);

  gtk_widget_show_all(GTK_WIDGET(mapsizewin));
  gtk_window_set_policy(GTK_WINDOW(mapsizewin), FALSE, FALSE, FALSE);

 } /* new_file() */


/****************************************************************************
button callbacks for file dialogs
*****************************************************************************/
void fs_cancel(GtkWidget *widget, GtkFileSelection * fs)
 {
  gtk_widget_destroy(GTK_WIDGET(fs));
 } /* open_cancel() */


void open_ok(GtkWidget *widget, GtkFileSelection *fs)
 {
  if(fs!=NULL)
   {
    if(filename==NULL)
      filename=(char *)malloc(256*sizeof(char *));
    strcpy(filename,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
    gtk_widget_destroy(GTK_WIDGET(fs));
   }
  load_game(filename);

  /* clear all infrastructure from the map */
  clear_rect_infrastructure(0, 0, map.xsize, map.ysize);

  init_undo();

  draw_everything(FALSE);
  refresh_screen();
 } /* open_ok() */


void save_ok(GtkWidget *widget, GtkFileSelection *fs)
 {
  if(filename==NULL)
    filename=(char *)malloc(256*sizeof(char *));
  strcpy(filename,gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
  save_game(filename);
  gtk_widget_destroy(GTK_WIDGET(fs));
 } /* save_ok() */


/****************************************************************************
open a freeciv save game
*****************************************************************************/
void open_file()
 {
  GtkWidget *odialog;

  odialog = gtk_file_selection_new("Open savegame...");
  gtk_window_set_modal(GTK_WINDOW(odialog), TRUE);
  if(filename!=NULL)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(odialog),filename);
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(odialog));
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(odialog)->cancel_button),
		     "clicked",(GtkSignalFunc) fs_cancel, odialog);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(odialog)->ok_button),
		     "clicked",(GtkSignalFunc) open_ok, odialog);

  gtk_widget_show(odialog);
 } /* open_file() */


/****************************************************************************
save the map as the current freeciv save file
*****************************************************************************/
void save_file()
 {
  if(filename==NULL)
    save_file_as();
  else
    save_game(filename);
 } /* save_file() */


/****************************************************************************
save a freeciv save game
*****************************************************************************/
void save_file_as()
 {
  GtkWidget *sdialog;
  GtkWidget *widget0, *widget1;
  GSList *group=NULL;
  char *labels[]={ "Specials and Positions", "Startpositions", "only Terrain" };
  gint i;

  sdialog=gtk_file_selection_new("Save scenario as...");
  if(filename!=NULL)
    gtk_file_selection_set_filename(GTK_FILE_SELECTION(sdialog),filename);
  gtk_window_set_modal(GTK_WINDOW(sdialog), TRUE);
  gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(sdialog));

  widget0=gtk_frame_new("Scenario Type");
  gtk_box_pack_start(GTK_BOX(GTK_FILE_SELECTION(sdialog)->action_area),
                      widget0, TRUE, TRUE, 0);
  gtk_widget_show(widget0);

  widget1=gtk_hbox_new(TRUE,10);
  gtk_container_add(GTK_CONTAINER(widget0), widget1);
  set_scenario();

  for(i=1;i<=3;i++)
   {
    widget0=gtk_radio_button_new_with_label(group, labels[i-1]);
    group=gtk_radio_button_group(GTK_RADIO_BUTTON(widget0));
    gtk_signal_connect(GTK_OBJECT(widget0), "toggled",
                       (GtkSignalFunc) scenario_toggled, GINT_TO_POINTER(i));
    if(game.scenario==i) {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widget0), TRUE);
      gtk_toggle_button_toggled(GTK_TOGGLE_BUTTON(widget0));
    } else {
      gtk_toggle_button_set_state(GTK_TOGGLE_BUTTON(widget0), FALSE);
    }
    if(game.scenario==3)
      gtk_widget_set_sensitive(GTK_WIDGET(widget0), FALSE);
    gtk_tooltips_set_tip(tooltips, widget0,
	"Select the data you wish to save in the "
        "scenario. You can save just the terrain map, "
        "the terrain map and starting positions, or "
        "terrain with specials and starting positions.", NULL);
    gtk_box_pack_start(GTK_BOX(widget1), widget0, FALSE, FALSE, 0);
    gtk_widget_show(widget0);
   }

  gtk_widget_show(widget1);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(sdialog)->cancel_button),
		     "clicked",(GtkSignalFunc) fs_cancel, sdialog);
  gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(sdialog)->ok_button),
		     "clicked",(GtkSignalFunc) save_ok, sdialog);

  gtk_widget_show(sdialog);
 } /* save_file_as() */


/****************************************************************************
sets the "Show Toolbox" menu item to off
*****************************************************************************/
void toggle_tool_off(void)
 {
  gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(toolmi),FALSE);
 } /* toggle_menuitem_off() */


/****************************************************************************
sets the "Show Toolbox" menu item to off
*****************************************************************************/
void toggle_over_off(void)
 {
  gtk_check_menu_item_set_state(GTK_CHECK_MENU_ITEM(overmi),FALSE);
 } /* toggle_menuitem_off() */


/****************************************************************************
sets game.scenario to the scenario type selected in the save-as-dialog
*****************************************************************************/
void scenario_toggled(GtkWidget *widget, gpointer scenario)
 {
  game.scenario=GPOINTER_TO_INT(scenario);
 } /* scenario_toggled() */
